package com.mx.dla.dda.reporte.pago.daos;

import java.util.Date;
import java.util.List;
import java.util.Map;

import com.mx.dla.dda.reporte.pago.dtos.DetalleContratoDTO;
import com.mx.dla.dda.reporte.pago.dtos.ReporteAddOnRowDTO;
import com.mx.dla.dda.reporte.pago.dtos.ReporteCRowDTO;
import com.mx.dla.dda.reporte.pago.dtos.ReporteInterfazContingenciaDTO;
import com.mx.dla.dda.reporte.pago.dtos.ReporteRowDTO;

public interface ReportePagoAlternoDAO {
	public List<DetalleContratoDTO> detalleDontratos();
	public List<ReporteRowDTO> obtenerReporteRows(Long idContrato, Long idEstudio, Map<String, String> parametros);
	public List<ReporteInterfazContingenciaDTO> obtenerReporteRowsC(String idContrato, Long idEstudio);
	public List<ReporteInterfazContingenciaDTO> obtenerReporteRowsEnmienda(String idContrato);
	public List<ReporteCRowDTO> obtenerReporteRowsCExcel(String idContrato, Long idEstudio) ;
	public List<Map<String, String>> obtenerListaMapas();
	public List<ReporteCRowDTO> obtenerReporteRowsS(Date fechaInicio);
	public List<ReporteAddOnRowDTO> obtenerReporteGeneral(Date fechaInicio,Long idEstudio, String notIn);
	public List<ReporteAddOnRowDTO> obtenerReporteGeneralHBO(Date fechaInicio,Long idEstudio, String notIn);
	public List<ReporteAddOnRowDTO> obtenerReporteGeneralArgentina(Date fechaInicio,Long idEstudio);
	public List<ReporteAddOnRowDTO> obtenerReporteGeneralMexico(Date fechaInicio,Long idEstudio);
	public List<ReporteAddOnRowDTO> obtenerReporteFoxGeneral(Date fechaInicio,Long idEstudio);
	
	public List<ReporteAddOnRowDTO> obtenerReporteKaraokeQello(Date fechaInicio,Long idEstudio, String notIn);
	public List<ReporteAddOnRowDTO> obtenerReporteKaraokeQelloArgentina(Date fechaInicio,Long idEstudio);
}
